///////////////////////////////////////////////////////////////////
// ADSInterrupt.h
// 720020-1149x
//
// Applied Data Systems
//
// This class enables easy hooking of ADS system supplied
// external interrupts.  The general method of use is to 
// call Connect with the interrupt of interest (defined 
// in ADSInterrupt.h), followed by Initialize with the 
// handle of an event that you want the interrupt to set.
// When handling the event, ensure you call Done as the 
// interrupt will be *disabled* until Done is called().
//
// History
// -------
// 20-13121: November 4, 2002 (ctacke)
//
// 720020-11492: May 1, 2003 (ctacke)
//  - renumbered
//  - Added BitsyX constants
//
// 720020-11493: May 14, 2003 (ctacke)
//  - Added GPIO constants
//  - Added SetGPIODirection() and modified Connect()
//
///////////////////////////////////////////////////////////////////

#include <windows.h>

///////////////////////////////////////////////////////////////////
/////////////  ADS SYSTEM INTERRUPTS FOR EXTERNAL USE /////////////
//
// The following constants can be used with ADSInterrupt::Connect
// The SIGNAL is passed as the InterruptNumber (param 1).
// The MODEs are OR'ed together and passed as InterruptMode (param 2)
//
// The SIGNAL is the signal based on the product user's manual.
// EDGE is whether or not the product supports edge interrupts
// LEVEL is whether or not the product supports level interrupts
// DIRECTION is the supported edge direction for the interrupt
// 
// It is recommended that *only these values* be used with any ADS
// product
//
// Also, it is IMPORTANT that you note the revision of your board, as
// revisions within the same product family may have routed signals
// differently.
///////////////////////////////////////////////////////////////////

#define INTR_MODE_EDGE              0x00000001
#define INTR_MODE_LEVEL             0x00000002
#define INTR_MODE_POSITIVE_LOGIC    0x00000004
#define INTR_MODE_NEGATIVE_LOGIC    0x00000008
#define INTR_MODE_POWERDOWN_WAKEUP  0x00000010


/////// INTERRUPT SIGNAL                EDGE  LEVEL LOGIC
/////// ------------------              ----  ----- ---------
// BitsyX Rev 2
#define BITSY_X_CARDBIRQ		115   //  Y     N    NEGATIVE
#define BITSY_X_EXT_IRQ1		58    //  Y     N    BOTH
#define BITSY_X_EXT_IRQ2		73    //  Y     N    BOTH
#define BITSY_X_EXT_IRQ3		33    //  Y     N    BOTH
/////// ------------------              ----  ----- ---------
// Bitsy Plus Rev 5
#define BITSY_PLUS_RQONOFF      2     //  Y     N    POSITIVE
#define BITSY_PLUS_CARDBIRQ     116   //  Y     N    NEGATIVE
#define BITSY_PLUS_EXT_IRQ1     75    //  Y     N    BOTH
#define BITSY_PLUS_EXT_IRQ2     74    //  Y     N    BOTH
#define BITSY_PLUS_EXT_IRQ3     11    //  Y     N    BOTH
/////// ------------------              ----  ----- ---------
// Bitsy Rev C
#define BITSY_RQONOFF	        2     //  Y     N    POSITIVE
#define BITSY_CARDBIRQ          116   //  Y     N    NEGATIVE
#define BITSY_EXT_IRQ1          82    //  Y     N    POSITIVE
#define BITSY_EXT_IRQ2          70    //  Y     N    POSITIVE
/////// ------------------              ----  ----- ---------
// Graphics Master Rev C
#define GMASTER_RQONOFF	        2     //  Y     N    POSITIVE
#define GMASTER_EXT_IRQ         57    //  Y     N    NEGATIVE
/////// ------------------              ----  ----- ---------
// Graphics Client Plus Rev D
#define GCPLUS_RQONOFF          2     //  Y     N    POSITIVE
#define GCPLUS_EXT_IRQ          57    //  Y     N    NEGATIVE  *** UNSUPPORTED ***

/////// ------------------              ----  ----- ---------
// Generic StrongARM GPIO interrupts - check your device docs for availability
#define SA1100_GPIO25_EDGE      47    //  Y		N    BOTH
#define SA1100_GPIO26_EDGE      48    //  Y		N    BOTH
#define SA1100_GPIO27_EDGE      49    //  Y		N    BOTH

class ADSInterrupt
{
private:
	typedef ULONG INTR_MODE;

	typedef enum _INTERFACE_TYPE 
	{
		InterfaceTypeUndefined = -1,
		Internal,
		Isa,
		Eisa,
		MicroChannel,
		TurboChannel,
		PCIBus,
		VMEBus,
		NuBus,
		PCMCIABus,
		CBus,
		MPIBus,
		MPSABus,
		ProcessorInternal,
		InternalPowerBus,
		PNPISABus,
		PNPBus,
		MaximumInterfaceType
	} INTERFACE_TYPE, *PINTERFACE_TYPE;

	typedef ULONG (*INTERRUPTCONNECT)(   
				IN INTERFACE_TYPE InterfaceType,
				IN ULONG BusNumber,
				IN ULONG BusInterrupt,
				IN INTR_MODE InterruptMode);

	typedef ULONG (*INTERRUPTDISCONNECT)(ULONG);  

	typedef BOOL (*INTERRUPTINITIALIZE) (
				DWORD idInt, 
				HANDLE hEvent, 
				LPVOID pvData, 
				DWORD cbData);

	typedef BOOL (*INTERRUPTDONE) (DWORD idInt);

	typedef VOID (*INTERRUPTDISABLE) (DWORD idInt);

	INTERRUPTCONNECT InterruptConnect;
	INTERRUPTDISCONNECT InterruptDisconnect;
	INTERRUPTINITIALIZE InterruptInitialize;
	INTERRUPTDONE InterruptDone;
	INTERRUPTDISABLE InterruptDisable;

	HINSTANCE	m_hBceddkDll;
	HINSTANCE	m_hCoreDll;
	BOOL		m_bInit;
	BOOL		m_bConnected;
	ULONG		m_hInterrupt;
	HANDLE		m_hIntEvent;

	void SetGPIODirection(int GPIO, BYTE Direction);

public:
	ADSInterrupt();
	~ADSInterrupt();
	
	BOOL Connect(ULONG InterruptNumber, INTR_MODE InterruptMode);
	long Initialize(HANDLE EventToSet);
	void Disable();
	long Done();
	long Disconnect();
};
